package ca.yorku.cse.mack.h4touch;

import android.app.Activity;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.CheckBox;
import android.widget.Spinner;
import android.widget.Toast;

public class H4TouchSetup extends Activity
{
	private Spinner spinParticipant;
	private Spinner spinSession, spinGroup, spinCondition;
	private Spinner spinPhrasesPerBlock, spinPhrasesFile;
	private Spinner spinKeyStyle;
	private CheckBox checkVibrotactileFeedback, checkAuditoryFeedback;
	private CheckBox checkShowText;
	private CheckBox checkLowercaseOnly;
	private CheckBox checkShowHuffman;

	/*
	 * The following arrays are used to fill the spinners in the set up dialog. The first entries will be replaced by
	 * corresponding values in the app's shared preferences, if any exist. In order for a value to exit as a shared
	 * preference, the app must have been run at least once with the "Save" button tapped.
	 */
	String[] participantCode = { "P01", "P01", "P02", "P03", "P04", "P05", "P06", "P07", "P08", "P09", "P10", "P11",
			"P12", "P13", "P14", "P15", "P16", "P17", "P18", "P19", "P20", "P21", "P22", "P23", "P24", "P25" };
	String[] sessionCode = { "S01", "S01", "S02", "S03", "S04", "S05", "S06", "S07", "S08", "S09", "S10", "S11", "S12",
			"S13", "S14", "S15", "S16", "S17", "S18", "S19", "S20", "S21", "S22", "S23", "S24", "S25" };
	String[] blockCode = { "(auto)" };
	String[] groupCode = { "G01", "G01", "G02", "G03", "G04", "G05", "G06", "G07", "G08", "G09", "G10", "G11", "G12",
			"G13", "G14", "G15", "G16", "G17", "G18", "G19", "G20", "G21", "G22", "G23", "G24", "G25" };
	String[] conditionCode = { "C01", "C01", "C02", "C03", "C04", "C05", "C06", "C07", "C08", "C09", "C10", "C11",
			"C12", "C13", "C14", "C15", "C16", "C17", "C18", "C19", "C20", "C21", "C22", "C23", "C24", "C25" };
	String[] phrasesPerBlockArray = { "5", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10" };
	String[] phrasesFileArray = { "phrases2", "phrases2", "quickbrownfox", "phrases100", "alphabet" };
	String[] keyStyleArray = { "+", "x", "+" };

	// defaults for booleans (may be different if shared preferences saved)
	boolean vibrotactileFeedback = true;
	boolean auditoryFeedback = false;
	boolean showText = true;
	boolean lowercaseOnly = true;
	boolean showHuffman = true;
	// end set up parameters

	SharedPreferences sp;
	SharedPreferences.Editor spe;

	/** Called when the activity is first created. */
	@Override
	public void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);
		setContentView(R.layout.setup);

		// initialize reference to shared preferences (values saved are default values)
		sp = this.getPreferences(MODE_PRIVATE);

		// overwrite 1st entry from shared preferences, if corresponding value exits
		participantCode[0] = sp.getString("participantCode", participantCode[0]);
		sessionCode[0] = sp.getString("sessionCode", sessionCode[0]);
		// block code initialized in main activity (based on existing filenames)
		groupCode[0] = sp.getString("groupCode", groupCode[0]);
		conditionCode[0] = sp.getString("conditionCode", conditionCode[0]);
		phrasesPerBlockArray[0] = sp.getString("phrasesPerBlock", phrasesPerBlockArray[0]);
		phrasesFileArray[0] = sp.getString("phrasesFile", phrasesFileArray[0]);
		keyStyleArray[0] = sp.getString("keyStyle", keyStyleArray[0]);
		vibrotactileFeedback = sp.getBoolean("vibrotactileFeedback", vibrotactileFeedback);
		auditoryFeedback = sp.getBoolean("auditoryFeedback", auditoryFeedback);
		showText = sp.getBoolean("showText", showText);
		lowercaseOnly = sp.getBoolean("lowercaseOnly", lowercaseOnly);
		showHuffman = sp.getBoolean("showHuffman", showHuffman);

		// get references to widgets in setup dialog
		spinParticipant = (Spinner)findViewById(R.id.paramPart);
		spinSession = (Spinner)findViewById(R.id.paramSess);
		Spinner spinBlock = (Spinner)findViewById(R.id.paramBlock);
		spinGroup = (Spinner)findViewById(R.id.paramGroup);
		spinCondition = (Spinner)findViewById(R.id.paramCondition);
		spinPhrasesPerBlock = (Spinner)findViewById(R.id.paramPhrasesPerBlock);
		spinPhrasesFile = (Spinner)findViewById(R.id.paramPhrasesFile);
		spinKeyStyle = (Spinner)findViewById(R.id.paramKeyStyle);
		checkVibrotactileFeedback = (CheckBox)findViewById(R.id.paramVibrotactileFeedback);
		checkAuditoryFeedback = (CheckBox)findViewById(R.id.paramAuditoryFeedback);
		checkShowText = (CheckBox)findViewById(R.id.paramShowText);
		checkLowercaseOnly = (CheckBox)findViewById(R.id.paramLowercase);
		checkShowHuffman = (CheckBox)findViewById(R.id.paramShowHuffman);

		// initialise spinner adapters
		ArrayAdapter<CharSequence> adapterPC = new ArrayAdapter<CharSequence>(this, R.layout.spinnerstyle,
				participantCode);
		spinParticipant.setAdapter(adapterPC);

		ArrayAdapter<CharSequence> adapterSS = new ArrayAdapter<CharSequence>(this, R.layout.spinnerstyle, sessionCode);
		spinSession.setAdapter(adapterSS);

		ArrayAdapter<CharSequence> adapterB = new ArrayAdapter<CharSequence>(this, R.layout.spinnerstyle, blockCode);
		spinBlock.setAdapter(adapterB);

		ArrayAdapter<CharSequence> adapterG = new ArrayAdapter<CharSequence>(this, R.layout.spinnerstyle, groupCode);
		spinGroup.setAdapter(adapterG);

		ArrayAdapter<CharSequence> adapterC = new ArrayAdapter<CharSequence>(this, R.layout.spinnerstyle, conditionCode);
		spinCondition.setAdapter(adapterC);

		ArrayAdapter<CharSequence> adapterPPB = new ArrayAdapter<CharSequence>(this, R.layout.spinnerstyle,
				phrasesPerBlockArray);
		spinPhrasesPerBlock.setAdapter(adapterPPB);

		ArrayAdapter<CharSequence> adapterPF = new ArrayAdapter<CharSequence>(this, R.layout.spinnerstyle,
				phrasesFileArray);
		spinPhrasesFile.setAdapter(adapterPF);

		// NOTE: different spinner style
		ArrayAdapter<CharSequence> adapterKO = new ArrayAdapter<CharSequence>(this, R.layout.spinnerstylelargetext,
				keyStyleArray);
		spinKeyStyle.setAdapter(adapterKO);

		checkVibrotactileFeedback.setChecked(vibrotactileFeedback);
		checkAuditoryFeedback.setChecked(auditoryFeedback);
		checkShowText.setChecked(showText);
		checkLowercaseOnly.setChecked(lowercaseOnly);
		checkShowHuffman.setChecked(showHuffman);
	}

	/** Called when the "OK" button is pressed. */
	public void clickOK(View view)
	{
		// get user's choices
		String part = participantCode[spinParticipant.getSelectedItemPosition()];
		String sess = sessionCode[spinSession.getSelectedItemPosition()];
		// String block = blockCode[spinBlock.getSelectedItemPosition()];
		String group = groupCode[spinGroup.getSelectedItemPosition()];
		String condition = conditionCode[spinCondition.getSelectedItemPosition()];
		int phrasesPerBlock = Integer.parseInt(phrasesPerBlockArray[spinPhrasesPerBlock.getSelectedItemPosition()]);
		String phrasesFile = phrasesFileArray[spinPhrasesFile.getSelectedItemPosition()];
		String keyStyle = keyStyleArray[spinKeyStyle.getSelectedItemPosition()];
		boolean vibrotactileFeedback = checkVibrotactileFeedback.isChecked();
		boolean auditoryFeedback = checkAuditoryFeedback.isChecked();
		boolean showText = checkShowText.isChecked();
		boolean lowercaseOnly = checkLowercaseOnly.isChecked();
		boolean showHuffmanCodes = checkShowHuffman.isChecked();

		Bundle b = new Bundle();
		b.putString("participantCode", part);
		b.putString("sessionCode", sess);
		// b.putString("blockCode", block);
		b.putString("groupCode", group);
		b.putString("conditionCode", condition);
		b.putInt("phrasesPerBlock", phrasesPerBlock);
		b.putString("phrasesFile", phrasesFile);
		b.putString("keyStyle", keyStyle);
		b.putBoolean("vibrotactileFeedback", vibrotactileFeedback);
		b.putBoolean("auditoryFeedback", auditoryFeedback);
		b.putBoolean("showText", showText);
		b.putBoolean("lowercaseOnly", lowercaseOnly);
		b.putBoolean("showHuffmanCodes", showHuffmanCodes);

		// start experiment activity
		Intent i = new Intent(getApplicationContext(), H4TouchActivity.class);
		i.putExtras(b);
		startActivity(i);
		finish();
	}

	/** Called when the "Save" button is pressed. */
	public void clickSave(View view)
	{
		spe = sp.edit();
		spe.putString("participantCode", participantCode[spinParticipant.getSelectedItemPosition()]);
		spe.putString("sessionCode", sessionCode[spinSession.getSelectedItemPosition()]);
		spe.putString("groupCode", groupCode[spinGroup.getSelectedItemPosition()]);
		spe.putString("conditionCode", conditionCode[spinCondition.getSelectedItemPosition()]);
		spe.putString("phrasesPerBlock", phrasesPerBlockArray[spinPhrasesPerBlock.getSelectedItemPosition()]);
		spe.putString("phrasesFile", phrasesFileArray[spinPhrasesFile.getSelectedItemPosition()]);
		spe.putString("keyStyle", keyStyleArray[spinKeyStyle.getSelectedItemPosition()]);
		spe.putBoolean("vibrotactileFeedback", checkVibrotactileFeedback.isChecked());
		spe.putBoolean("auditoryFeedback", checkAuditoryFeedback.isChecked());
		spe.putBoolean("showText", checkShowText.isChecked());
		spe.putBoolean("lowercaseOnly", checkLowercaseOnly.isChecked());
		spe.putBoolean("showHuffman", checkShowHuffman.isChecked());
		spe.apply();
		Toast.makeText(this, "Preferences saved!", Toast.LENGTH_SHORT).show();
	}

	/** Called when the "Exit" button is pressed. */
	public void clickExit(View view)
	{
		super.onDestroy(); // cleanup
		this.finish(); // terminate
	}
}
